#!/usr/bin/env python
#
#   jsonutil.py
#   pinet
#
#   Copyright (C) 2008 Ross Light
#
#   This file is part of zfilenet.
#
#   zfilenet is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   zfilenet is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with zfilenet.  If not, see <http://www.gnu.org/licenses/>.
#

"""Utilities for dealing with JSON"""

import random
import socket

import simplejson
import zope.interface

__author__ = 'Ross Light'
__date__ = 'October 14, 2007'
__docformat__ = 'reStructuredText'
__all__ = ['IDictSerializer',
           'IDictReader',
           'IDictWriter',
           'DictSerializerDecoder',
           'DictSerializerEncoder',]

class_key = '__class__'

class IDictSerializer(zope.interface.Interface):
    def get_json_name():
        """
        Retrieves the code name that should be serialized.
        
        :Returns: The code name of the object
        :ReturnType: str
        """

class IDictReader(IDictSerializer):
    def fromdict(d):
        """
        Loads an object from a dictionary.
        
        :Returns: The loaded object
        :ReturnType: object
        """

class IDictWriter(IDictSerializer):
    def todict():
        """
        Creates a dictionary from an object.
        
        :Returns: The archived dictionary
        :ReturnType: dict
        """

class DictSerializerDecoder(simplejson.JSONDecoder):
    """Decodes objects that implement the `IDictReader` interface."""
    
    def __init__(self, encoding=None, object_hook=None, *args, **kw):
        super(DictSerializerDecoder, self).__init__(encoding,
                                                    self._object_handler,
                                                    *args, **kw)
        self.__readers = {}
        self.__object_hook = object_hook
    
    def add_reader(self, reader):
        """
        Adds a reader to the decoder.
        
        :Parameters:
            reader : provider of `IDictReader`
                The reader to add
        :Raises TypeError: If reader does not provide `IDictReader`
        :Raises ValueError: If reader is already in decoder
        """
        # Check reader argument
        if not IDictReader.providedBy(reader):
            raise TypeError("Reader does not provide IDictReader")
        # Put reader into dictionary
        name = reader.get_json_name()
        if name not in self.__readers:
            self.__readers[name] = reader
        else:
            raise ValueError("Reader is already in decoder")
    
    def remove_reader(self, reader):
        """
        Removes a reader from the decoder.
        
        :Parameters:
            reader : string or provider of `IDictReader`
                The reader to remove, or the reader's code name
        :Raises TypeError: If reader is not of the correct type
        :Raises ValueError: If reader is not in the decoder
        """
        # Find name
        if isinstance(reader, basestring):
            name = reader
        elif IDictReader.providedBy(reader):
            name = reader.get_json_name()
        else:
            raise TypeError("Reader does not provide IDictReader")
        # Remove from readers dictionary
        try:
            del self.__readers[name]
        except KeyError:
            raise ValueError("Reader is not in decoder")
    
    def _object_handler(self, obj):
        try:
            cls = obj[class_key]
            reader = self.__readers[cls]
        except KeyError:
            if self.__object_hook is None:
                return obj
            else:
                return self.__object_hook(obj)
        else:
            return reader.fromdict(obj)

class DictSerializerEncoder(simplejson.JSONEncoder):
    """Decodes objects that implement the `IDictReader` interface."""
    def default(self, obj):
        if IDictWriter.providedBy(obj):
            d = obj.todict()
            d[class_key] = obj.get_json_name()
            return d
        else:
            return super(DictSerializerEncoder, self).default(obj)
