#!/usr/bin/env python
#
#   zfilenet/db.py
#   zfilenet
#
#   Copyright (C) 2008 Ross Light
#
#   This file is part of zfilenet.
#
#   zfilenet is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   zfilenet is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with zfilenet.  If not, see <http://www.gnu.org/licenses/>.
#

"""Several generic database utilities"""

import os

from sqlalchemy import (Table, Column, MetaData, create_engine,
                        Integer, String, ForeignKey)
from sqlalchemy.orm import mapper, sessionmaker, synonym

__author__ = 'Ross Light'
__date__ = 'June 13, 2008'
__all__ = ['basic_metadata',
           'DBVariable',
           'connect_sqlite3',]

### DATABASE VARIABLES ###

basic_metadata = MetaData()
db_variables_table = Table('db_variables', basic_metadata,
    Column('variable_id', Integer, primary_key=True),
    Column('variable_key', String(255), unique=True),
    Column('variable_value', String(512)),
)

class DBVariable(object):
    """
    A variable in the database.
    
    Database variables work like a dictionary; they are mostly used as bits of
    miscellaneous information about the data (like data version).
    
    :IVariables:
        key : str
            The variable's name
        value : str
            The variable's value
    """
    @classmethod
    def get_variable(cls, session, key, *args):
        """
        Retrieves a variable's value.
        
        If an additional argument is given, then it is returned if the variable
        was not found.
        
        :Parameters:
            session : sqlalchemy.orm.session.Session
                The session to make queries from
            key : str
                The name of the variable
        :Raises KeyError: If the variable is not found and no default is
                          provided
        :Returns: The value of the variable
        :ReturnType: str
        """
        # Determine default
        if len(args) == 0:
            has_default, default = False, None
        elif len(args) == 1:
            has_default, default = True, args[0]
        else:
            raise TypeError("get_variable takes at most 3 arguments")
        # Get variable
        result = session.query(cls).filter_by(key=key).first()
        if result is not None:
            return result.value
        elif has_default:
            return default
        else:
            raise KeyError(key)
    
    @classmethod
    def set_variable(cls, session, key, value):
        """
        Modifies a variable's value.
        
        :Parameters:
            session : sqlalchemy.orm.session.Session
                The session to make queries from
            key : str
                The name of the variable
            value : str
                The new value for the variable
        """
        result = session.query(cls).filter_by(key=key).first()
        if result is not None:
            result.value = value
        else:
            new_variable = cls(key, value)
            session.save(new_variable)
        session.commit()
    
    @classmethod
    def get_variable_dict(cls, session):
        """
        Collects all the database's variables into a dictionary.
        
        :Parameters:
            session : sqlalchemy.orm.session.Session
                The session to make queries from
        :Returns: The variables dictionary
        :ReturnType: dict
        """
        results = session.query(cls)
        result_dict = {}
        for variable in results:
            result_dict[variable.key] = variable.value
        return result_dict
    
    def __init__(self, key, value):
        self.key, self.value = key, value

mapper(DBVariable, db_variables_table, properties=dict(
    key=db_variables_table.c.variable_key,
    value=db_variables_table.c.variable_value,
))

### FUNCTIONS ###

def connect_sqlite3(metadata, path, new_hook=None):
    """
    Connects to a SQLite3 database, creating the schema if necessary.
    
    :Parameters:
        metadata : sqlalchemy.schema.MetaData
            SQLAlchemy table metadata
        path : str
            The database's path
        new_hook
            A callback used if the database is to be newly created
    :Returns: The connected database engine
    :ReturnType: sqlalchemy.engine.Engine
    """
    db_exists = os.path.exists(path)
    engine = create_engine('sqlite:///' + path) # yes, three slashes
    if path == ':memory:' or not db_exists:
        basic_metadata.create_all(engine)
        metadata.create_all(engine)
        if new_hook is not None:
            new_hook(engine)
    return engine
